from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from fileinput import filename

import os
from torch.utils.data import Dataset
import numpy as np
import pickle
from dataloaders.rawvideo_util import RawVideoExtractor
from dataloaders.dataloader_msvd_retrieval import MSVD_DataLoader
from os.path import exists
import time
from modules.tokenization_clip import SimpleTokenizer as ClipTokenizer

class MSVD_extractor(MSVD_DataLoader):
    def __init__(self, subset, data_path, features_path, tokenizer, max_words=32, feature_framerate=1, max_frames=12, image_resolution=224, frame_order=0, slice_framepos=2):
        super().__init__(subset, data_path, features_path, tokenizer, max_words, feature_framerate, max_frames, image_resolution, frame_order, slice_framepos)
        self.save_dir = "/home/jeff/DeepLearning/MSVD/msvd_data/npz_videos"
        if not exists(self.save_dir):   os.mkdir(self.save_dir)

    def extract(self, choice_video_ids):
        

        video_mask = np.zeros((len(choice_video_ids), self.max_frames), dtype=np.long)
        max_video_length = [0] * len(choice_video_ids)

        # Pair x L x T x 3 x H x W
        video = np.zeros((len(choice_video_ids), self.max_frames, 1, 3,
                          self.rawVideoExtractor.size, self.rawVideoExtractor.size), dtype=np.float)

        for i, video_id in enumerate(choice_video_ids):
            try:
                video_path = self.video_dict[video_id]
            except:
                print("Video path not found: ", video_id)
                return 

            raw_video_data = self.rawVideoExtractor.get_video_data(video_path)
            raw_video_data = raw_video_data['video']

            if len(raw_video_data.shape) > 3:
                raw_video_data_clip = raw_video_data
                # L x T x 3 x H x W
                raw_video_slice = self.rawVideoExtractor.process_raw_data(raw_video_data_clip)
                if self.max_frames < raw_video_slice.shape[0]:
                    if self.slice_framepos == 0:
                        video_slice = raw_video_slice[:self.max_frames, ...]
                    elif self.slice_framepos == 1:
                        video_slice = raw_video_slice[-self.max_frames:, ...]
                    else:
                        sample_indx = np.linspace(0, raw_video_slice.shape[0] - 1, num=self.max_frames, dtype=int)
                        video_slice = raw_video_slice[sample_indx, ...]
                else:
                    video_slice = raw_video_slice

                video_slice = self.rawVideoExtractor.process_frame_order(video_slice, frame_order=self.frame_order)

                slice_len = video_slice.shape[0]
                max_video_length[i] = max_video_length[i] if max_video_length[i] > slice_len else slice_len
                if slice_len < 1:
                    pass
                else:
                    video[i][:slice_len, ...] = video_slice
            else:
                print("video path: {} error. video id: {}".format(video_path, video_id))

        for i, v_length in enumerate(max_video_length):
            video_mask[i][:v_length] = [1] * v_length


            video_info = {
                "video":video,
                "video_mask":video_mask
                        }
        save_path = os.path.join(self.save_dir, video_id)
        np.savez_compressed(save_path, **video_info)
        print("Save file to ", save_path)



if __name__ == "__main__":
    start = time.time()

    tokenizer = ClipTokenizer()
    data_path = "/home/jeff/DeepLearning/MSVD/msvd_data"
    features_path = "/home/jeff/DeepLearning/MSVD/msvd_data/MSVD_Videos"
    subset = "val"

    extractor = MSVD_extractor(subset, data_path, features_path, tokenizer)

    cnt = 0

    for path, dir_list, file_list in os.walk("/home/jeff/DeepLearning/MSVD/msvd_data/MSVD_Videos"):
        for file_name in file_list:
            f_name = file_name.split('.')[0]
            choice_video_ids = [f_name]
            extractor.extract(choice_video_ids)
            # cnt += 1
            # if cnt > 10:    break

    print("Time cost:", (time.time() - start) / 60, "mins")
